<?php
/**
 * pnp4nagios–0.6.3
 * This file acts as the "front controller" to your application. You can
 * configure your application, modules, and system directories here.
 * PHP error_reporting level may also be changed.
 *
 * @see http://kohanaphp.com
 */

# fe, Tue, 20 Jul 2010 10:51:28 +0200
# check for access permissions for graphing a host/service
#
# must be authorized, otherwise exit
if(!isset($_SERVER['PHP_AUTH_USER'])) {
    echo 'Not authorized: no auth_user supplied!';
    exit;
} else {
    $user = htmlspecialchars($_SERVER['PHP_AUTH_USER']);
}

# check for authorized_for_all_hosts in nagios/cgi.cfg;
# authorized_for_all_services exists as well, but won't be checked extra
#
# not PHP per se, but said to be the fastest way ...
$haystack = "/etc/nagios3/cgi.cfg";
$needle = "authorized_for_all_hosts";
$handle = popen("grep --max-count=1 ^$needle $haystack", "r");
$output = fread($handle, 8192);
pclose($handle);
$output = str_replace("$needle=", "", $output);

# if the authorized user is not in authorized_for_all_hosts, check
# nagios permissions via socket
if (!in_array($user, explode(",", $output))) {
    # basic definitions: socket and query
    $livestatus = "/var/log/nagios3/rw/livestatus";
    $query  = <<<EOT
GET hosts
KeepAlive: on
Columns: host_name
AuthUser: $user

GET services
Columns: host_name description
AuthUser: $user

EOT;

    # connect to livestatus and submit our query
    $socket = socket_create(AF_UNIX, SOCK_STREAM, 0);
    if ($socket === false) {
	echo "socket_create() failed: reason: " . socket_strerror(socket_last_error()) . "\n";
	exit;
    }
    $result = socket_connect($socket, $livestatus);
    if ($result === false) {
	echo "socket_connect() failed: reason: ($result) " . socket_strerror(socket_last_error($socket)) . "\n";
	exit;
    }
    socket_write($socket, $query, strlen($query));
    # close for writing, thus signalling end of input
    socket_shutdown($socket, 1);

    # fill $output with a list of allowed hosts/services
    $output = '';
    $out = '';
    while ($out = socket_read($socket, 2048)) {
	$output .= $out;
    }
    socket_close($socket);

    # now exit if the desired host to display is not in the list of
    # allowed hosts/services in $output
    $targethost = htmlspecialchars($_GET['host']);
    $targetservice = $targethost . ';' . htmlspecialchars($_GET['srv']);
    if ((!in_array($targethost, explode("\n", $output))) and
      (!in_array($targetservice, explode("\n", $output)))) {
	echo "Not authorized for this view!\n";
	exit;
    }
}
# ok, now proceed normally
# end of addition by fe

/**
 * Define the website environment status. When this flag is set to TRUE, some
 * module demonstration controllers will result in 404 errors. For more information
 * about this option, read the documentation about deploying Kohana.
 *
 * @see http://docs.kohanaphp.com/installation/deployment
 */
define('IN_PRODUCTION', TRUE);

/**
* The current XML Structure Tag for this PNP Version
*/
define('XML_STRUCTURE_VERSION', '4');

/**
 * Website application directory. This directory should contain your application
 * configuration, controllers, models, views, and other resources.
 *
 * This path can be absolute or relative to this file.
 */
$kohana_application = 'application';

/**
 * Kohana modules directory. This directory should contain all the modules used
 * by your application. Modules are enabled and disabled by the application
 * configuration file.
 *
 * This path can be absolute or relative to this file.
 */
$kohana_modules = 'modules';

/**
 * Kohana system directory. This directory should contain the core/ directory,
 * and the resources you included in your download of Kohana.
 *
 * This path can be absolute or relative to this file.
 */
$kohana_system = '/usr/share/php/kohana/system';

/**
 * Test to make sure that Kohana is running on PHP 5.2 or newer. Once you are
 * sure that your environment is compatible with Kohana, you can comment this
 * line out. When running an application on a new server, uncomment this line
 * to check the PHP version quickly.
 */
version_compare(PHP_VERSION, '5.2', '<') and exit('Kohana requires PHP 5.2 or newer.');

/**
 * Set the error reporting level. Unless you have a special need, E_ALL is a
 * good level for error reporting.
 */
error_reporting(E_ALL & ~E_STRICT);

/**
 * Turning off display_errors will effectively disable Kohana error display
 * and logging. You can turn off Kohana errors in application/config/config.php
 */
ini_set('display_errors', TRUE);

/**
 * If you rename all of your .php files to a different extension, set the new
 * extension here. This option can left to .php, even if this file has a
 * different extension.
 */
define('EXT', '.php');

//
// DO NOT EDIT BELOW THIS LINE, UNLESS YOU FULLY UNDERSTAND THE IMPLICATIONS.
// ----------------------------------------------------------------------------
//

// Force default timezone
if(function_exists("date_default_timezone_set") and function_exists("date_default_timezone_get")){
	@date_default_timezone_set(@date_default_timezone_get());
}

$kohana_pathinfo = pathinfo(__FILE__);
// Define the front controller name and docroot
define('DOCROOT', $kohana_pathinfo['dirname'].DIRECTORY_SEPARATOR);
define('KOHANA',  $kohana_pathinfo['basename']);

// If the front controller is a symlink, change to the real docroot
is_link(KOHANA) and chdir(dirname(realpath(__FILE__)));

// If kohana folders are relative paths, make them absolute.
$kohana_application = file_exists($kohana_application) ? $kohana_application : DOCROOT.$kohana_application;
$kohana_modules = file_exists($kohana_modules) ? $kohana_modules : DOCROOT.$kohana_modules;
$kohana_system = file_exists($kohana_system) ? $kohana_system : DOCROOT.$kohana_system;

// Define application and system paths
define('APPPATH', str_replace('\\', '/', realpath($kohana_application)).'/');
define('MODPATH', str_replace('\\', '/', realpath($kohana_modules)).'/');
define('SYSPATH', str_replace('\\', '/', realpath($kohana_system)).'/');

// Clean up
unset($kohana_application, $kohana_modules, $kohana_system);

if (file_exists(DOCROOT.'install'.EXT) && !file_exists(DOCROOT.'install.ignore')){
	// Load the installation tests
	include DOCROOT.'install'.EXT;
}
else
{
	// Initialize Kohana
	require SYSPATH.'core/Bootstrap'.EXT;
}


